@@ -10,9 +10,12 @@ use Cwd qw/abs_path cwd/;
 use lib "tools/lib";
 use NQP::Configure qw(cmp_rev read_parrot_config gen_moar
                       fill_template_file fill_template_text
+                      probe_node
                       slurp system_or_die verify_install sorry gen_parrot);
 
-my %known_backends = (parrot => 1, jvm => 1, moar => 1);
+my @known_backends = qw/moar jvm parrot js/;
+my %known_backends = map { $_, 1; } @known_backends;
+my %prefixes = ( moar => 'm', jvm => 'j', parrot => 'p', js => 'js' );
 MAIN: {
     if (-r "config.default") {
         unshift @ARGV, shellwords(slurp('config.default'));
@@ -26,11 +29,14 @@ MAIN: {
 
     my %options;
     GetOptions(\%options, 'help!', 'prefix=s',
-               'with-parrot=s', 'gen-parrot:s',
+               'sysroot=s', 'sdkroot=s',
+               'backends=s',
+               'no-clean',
+               'with-parrot=s', 'gen-parrot:s', 'parrot-config=s', 'parrot-option=s@',
+               'with-moar=s', 'gen-moar:s', 'moar-option=s@',
                'make-install!', 'makefile-timing!',
-               'backends=s', 'gen-moar:s', 'moar-option=s@',
                'git-protocol=s',
-               'parrot-config=s', 'parrot-option=s@');
+               'git-depth=s', 'git-reference=s',);
 
     # Print help if it's requested
     if ($options{'help'}) {
@@ -45,9 +51,10 @@ MAIN: {
     }
 
     my $default_backend;
+    my @backends;
     my %backends;
     if ($options{backends}) {
-        $options{backends} = 'parrot,jvm,moar' if lc($options{backends}) eq 'all';
+        $options{backends} = join ',', @known_backends if lc($options{backends}) eq 'all';
         for my $be (split /,/, $options{backends}) {
             $be = lc $be;
             unless ($known_backends{$be}) {
@@ -55,26 +62,37 @@ MAIN: {
                     join(', ', sort keys %known_backends) . "\n";
             }
             $default_backend ||= $be;
+            push @backends, $be unless $backends{$be};
             $backends{$be} = 1;
         }
     }
-    if (defined $options{'gen-parrot'}) {
-        $backends{parrot} = 1;
-        $default_backend ||= 'parrot';
-    }
     if (defined $options{'gen-moar'}) {
+        push @backends, 'moar' unless $backends{moar};
         $backends{moar} = 1;
         $default_backend ||= 'moar';
     }
+    if (defined $options{'gen-parrot'}) {
+        push @backends, 'parrot' unless $backends{parrot};
+        $backends{parrot} = 1;
+        $default_backend ||= 'parrot';
+    }
     unless (%backends) {
         # TODO: come up with more sensible defaults
-        $backends{parrot} = 1;
-        $default_backend = 'parrot';
+        $backends{moar} = 1;
+        push @backends, 'moar';
+        $default_backend = 'moar';
     }
 
+    # XXX mkpath instead?
     mkdir($options{'prefix'}) if $options{'prefix'} && $^O =~ /Win32/ && !-d $options{'prefix'};
-    my $prefix      = ($options{'prefix'} && abs_path($options{'prefix'})) || cwd().'/install';
+    my $prefix      = $options{'prefix'}
+                    ? abs_path($options{'prefix'})
+                    : ($options{sysroot}
+		      ? '/usr'
+		      : File::Spec->catdir(cwd, 'install'));
     $config{prefix} = $prefix;
+    $config{sysroot} = $options{sysroot};
+    $config{sdkroot} = $options{sdkroot};
 
     # Save options in config.status
     unlink('config.status');
@@ -93,12 +111,13 @@ MAIN: {
     open my $MAKEFILE, '>', 'Makefile'
         or die "Cannot open 'Makefile' for writing: $!";
 
-    my @prefixes = sort map substr($_, 0, 1), keys %backends;
+    my @prefixes = map { $prefixes{$_} } @backends;
     print $MAKEFILE "\n# Makefile code generated by Configure.pl:\n";
 
-    my $launcher = substr($default_backend, 0, 1) . '-runner-default';
+    my $launcher = $prefixes{$default_backend} . '-runner-default';
     print $MAKEFILE "all: ", join(' ', map("$_-all", @prefixes), $launcher), "\n";
-    for my $t (qw/clean test qregex-test install/) {
+    print $MAKEFILE "install: ", join(' ', map("$_-install", @prefixes), $launcher . '-install'), "\n";
+    for my $t (qw/clean test qregex-test/) {
         print $MAKEFILE "$t: ", join(' ', map "$_-$t", @prefixes), "\n";
     }
 
@@ -210,7 +229,8 @@ MAIN: {
                 "(You can get a MoarVM built automatically with --gen-moar.)";
         }
         sorry(@errors) if @errors;
-        $config{'make'}   = $^O eq 'MSWin32' ? 'nmake' : 'make';
+        $config{'make'} = `$moar_path --libpath="src/vm/moar/stage0" "src/vm/moar/stage0/nqp.moarvm" -e "print(nqp::backendconfig()<make>)"`
+                        || 'make';
         $config{moar} = $moar_path;
         fill_template_file(
             'tools/build/Makefile-Moar.in',
@@ -219,6 +239,24 @@ MAIN: {
         );
     }
 
+    if ($backends{js}) {
+        $config{'make'}   = $^O eq 'MSWin32' ? 'nmake' : 'make';
+        my $node   = probe_node();
+
+        if ($node eq 'nodejs') {
+            sorry('You have a broken node.js. Please install node.js as node instead of nodejs.')
+        }
+        elsif (!$node) {
+            sorry("You don't have node.js. Please install node.js.");
+        }
+
+        fill_template_file(
+            'tools/build/Makefile-JS.in',
+            $MAKEFILE,
+            %config,
+        );
+    }
+
     if ($backends{jvm}) {
         my @errors;
 
@@ -306,6 +344,9 @@ Configure.pl - NQP Configure
 General Options:
     --help             Show this text
     --prefix=dir       Install files in dir
+    --sdkroot=dir      When given, use for searching build tools here, e.g.
+                       nqp, java etc.
+    --sysroot=dir      When given, use for searching runtime components here
     --backends=list    Backends to use: $backends
     --with-parrot=path/to/bin/parrot
                        Parrot executable to use to build NQP
@@ -318,6 +359,13 @@ General Options:
                        Options to pass to MoarVM configuration for --gen-moar
     --git-protocol={ssh,https,git}
                        Protocol to use for git clone. Default: https
+    --make-install     Immediately run `MAKE install` after configuring
+    --git-depth=<number>
+                       Use the --git-depth option for git clone with parameter number
+    --git-reference=<path>
+                       Use --git-reference option to identify local path where git repositories are stored
+                       For example: --git-reference=/home/user/repo/for_perl6
+                       Folders 'nqp', 'MoarVM', 'parrot' with corresponding git repos should be in for_perl6 folder
 
 Configure.pl also reads options from 'config.default' in the current directory.
 END
