@@ -4,6 +4,7 @@ import java.io.File;
 import java.lang.invoke.MethodHandle;
 import java.lang.invoke.MethodHandles;
 import java.lang.invoke.MethodType;
+import java.lang.reflect.Array;
 import java.lang.reflect.Constructor;
 import java.lang.reflect.Field;
 import java.lang.reflect.Method;
@@ -18,6 +19,7 @@ import java.util.List;
 import java.util.Map;
 
 import org.perl6.nqp.sixmodel.STable;
+import org.perl6.nqp.sixmodel.TypeObject;
 import org.perl6.nqp.sixmodel.SixModelObject;
 import org.perl6.nqp.sixmodel.StorageSpec;
 import org.perl6.nqp.sixmodel.reprs.JavaObjectWrapper;
@@ -47,7 +49,7 @@ public class BootJavaInterop {
     protected GlobalContext gc;
     
     /** If we need to load stuff from a JAR, the class loader for doing so. */
-    private HashMap<String, URLClassLoader> jarClassLoaders;
+    protected HashMap<String, URLClassLoader> jarClassLoaders;
 
     /** Create a new interop object for a context. */
     public BootJavaInterop(GlobalContext gc) {
@@ -94,29 +96,15 @@ public class BootJavaInterop {
 
     /** Main entry point for OO-ish callouts. */
     public SixModelObject typeForName(String name) {
+        Class<?> klass = null;
+        ClassLoader syscl = ClassLoader.getSystemClassLoader();
         try {
-            return getSTableForClass(Class.forName(name)).WHAT;
-        } catch (ClassNotFoundException e) {
-            throw ExceptionHandling.dieInternal(gc.getCurrentThreadContext(), e);
+            klass = syscl.loadClass(name);
         }
-    }
-    public SixModelObject typeForNameFromJAR(String name, String JAR) {
-        try {
-            URLClassLoader cl = jarClassLoaders.get(JAR);
-            if (cl == null) {
-                URL url = new URL("jar:" + new File(JAR).toURI().toURL() + "!/");
-                cl = new URLClassLoader(new URL[] { url });
-                jarClassLoaders.put(JAR, cl);
-            }
-            return getSTableForClass(Class.forName(name, true, cl)).WHAT;
-        } catch (ClassNotFoundException e) {
-            throw ExceptionHandling.dieInternal(gc.getCurrentThreadContext(), e);
-        } catch (MalformedURLException e) {
-            throw ExceptionHandling.dieInternal(gc.getCurrentThreadContext(), e);
-        } catch (Throwable e) {
-            e.printStackTrace();
-            throw e;
+        catch( ClassNotFoundException cnfe ) {
+            throw ExceptionHandling.dieInternal(gc.getCurrentThreadContext(), cnfe);
         }
+        return getSTableForClass(klass).WHAT;
     }
 
     /** Convenience methods for NQP coding. */
@@ -543,6 +531,135 @@ public class BootJavaInterop {
         emitPutToNQP(c, ix, storageForType(what));
     }
 
+    public static Object castObjectToClass(Object obj, Class<?> klass) throws Throwable {
+        Object retVal = null;
+
+        if( klass.isAssignableFrom(obj.getClass()) ) {
+            retVal = obj;
+        }
+        else if( klass.equals(Boolean.class) && obj.getClass().equals(Long.class) ){
+            retVal = new Boolean( ((Long)obj) == 0 ? false : true);
+        }
+        else if( klass.equals(Byte.class) && obj.getClass().equals(Long.class) ){
+            retVal = new Byte( ((Long)obj).byteValue() );
+        }
+        else if( klass.equals(Short.class) && obj.getClass().equals(Long.class) ){
+            retVal = new Short( ((Long)obj).shortValue() );
+        }
+        else if( klass.equals(Integer.class) && obj.getClass().equals(Long.class) ){
+            retVal = new Integer( ((Long)obj).intValue() );
+        }
+        else if( klass.equals(Long.class) && obj.getClass().equals(Long.class) ){
+            retVal = (Long) obj;
+        }
+        else if( klass.equals(Float.class) && obj.getClass().equals(Double.class) ){
+            retVal = new Float( ((Double)obj).floatValue() );
+        }
+        else if( klass.equals(Double.class) && obj.getClass().equals(Double.class) ){
+            retVal = (Double) obj;
+        }
+        else if( klass.equals(Character.class) && obj.getClass().equals(String.class) ){
+            retVal = new Character( ((String)obj).charAt(0));
+        }
+        else if( klass.equals(String.class) && obj.getClass().equals(String.class) ){
+            retVal = (String) obj;
+        }
+        else if( klass.equals(boolean.class) && obj.getClass().equals(Long.class) ) {
+            retVal = obj != null
+                ? ((Long) obj) == 0
+                    ? new Boolean(false)
+                    : new Boolean(true)
+                : null;
+        }
+        else if( klass.equals(byte.class) && obj.getClass().equals(Long.class) ) {
+            retVal = obj != null ? ((Long)obj).byteValue() : null;
+        }
+        else if( klass.equals(short.class) && obj.getClass().equals(Long.class) ) {
+            retVal = obj != null ? ((Long)obj).shortValue() : null;
+        }
+        else if( klass.equals(int.class) && obj.getClass().equals(Long.class) ) {
+            retVal = obj != null ? ((Long)obj).intValue() : null;
+        }
+        else if( klass.equals(long.class) && obj.getClass().equals(Long.class) ) {
+            retVal = obj;
+        }
+        else if( klass.equals(char.class) && obj.getClass().equals(String.class) ) {
+            retVal = ((String) obj).charAt(0);
+        }
+        else if( klass.equals(float.class) && obj.getClass().equals(Double.class) ) {
+            retVal = obj != null ? ((Double)obj).floatValue() : null;
+        }
+        else if( klass.equals(double.class) && obj.getClass().equals(Double.class) ) {
+            retVal = obj;
+        }
+
+        return retVal;
+    }
+
+    public static Object marshalOutRecursive(SixModelObject in, ThreadContext tc, Class<?> what) throws Throwable {
+        Object out = null;
+        int size = (int) Ops.elems(in, tc);
+        if(what != null) {
+            out = Array.newInstance(what.getComponentType(), size);
+            for( int i = 0; i < size; ++i ) {
+                in.at_pos_native(tc, i);
+                Object value = null;
+                if( tc.native_type == ThreadContext.NATIVE_NUM ) {
+                    value = new Double(tc.native_n);
+                }
+                else if( tc.native_type == ThreadContext.NATIVE_STR ) {
+                    value = tc.native_s;
+                }
+                else if( tc.native_type == ThreadContext.NATIVE_INT ) {
+                    value = new Long(tc.native_i);
+                } 
+                else {
+                    SixModelObject cur = Ops.atpos(in, i, tc);
+                    if( cur instanceof JavaObjectWrapper ) {
+                        value = RuntimeSupport.unboxJava(cur);
+                    } // XXX: probably cases missing here
+                    else {
+                        value = marshalOutRecursive(cur, tc, what.getComponentType());
+                    }
+                }
+                value = castObjectToClass(value, what.getComponentType());
+                Array.set(out, i, value);
+            }
+        }
+        else { 
+            // we've hopefully been called from a subclass, so we rely on them for casting and just unbox
+            for( int i = 0; i < size; ++i ) {
+                in.at_pos_native(tc, i);
+                Object value = null;
+                if( tc.native_type == ThreadContext.NATIVE_NUM ) {
+                    if( out == null ) 
+                        out = new Double[size];
+                    ((Double[]) out)[i] = new Double(tc.native_n);
+                }
+                else if( tc.native_type == ThreadContext.NATIVE_STR ) {
+                    if( out == null )
+                        out = new String[size];
+                    ((String[]) out)[i] = tc.native_s;
+                }
+                else if( tc.native_type == ThreadContext.NATIVE_INT ) {
+                    if( out == null )
+                        out = new Long[size];
+                    ((Long[]) out)[i] = new Long(tc.native_i);
+                } 
+                else {
+                    SixModelObject cur = Ops.atpos(in, i, tc);
+                    if( cur instanceof JavaObjectWrapper ) {
+                        ((Object[]) out)[i] = RuntimeSupport.unboxJava(cur);
+                    } // XXX: probably cases missing here?
+                    else {
+                        ((Object[]) out)[i] = marshalOutRecursive(cur, tc, null);
+                    }
+                }
+            }
+        }
+        return out;
+    }
+
     /**
      * Generates code for a marshal-out (NQP to Java).
      */
@@ -601,11 +718,26 @@ public class BootJavaInterop {
             mv.visitTypeInsn(Opcodes.CHECKCAST, Type.getInternalName(what));
             mv.visitLabel(done);
         }
-        else {
+        // array cases
+        else if(what.getComponentType() != null) {
             mv.visitVarInsn(Opcodes.ALOAD, c.tcLoc);
-            mv.visitMethodInsn(Opcodes.INVOKESTATIC, "org/perl6/nqp/runtime/Ops", "decont", Type.getMethodDescriptor(TYPE_SMO, TYPE_SMO, TYPE_TC));
+            mv.visitLdcInsn(Type.getType(what));
+            mv.visitMethodInsn(Opcodes.INVOKESTATIC, "org/perl6/nqp/runtime/BootJavaInterop", "marshalOutRecursive",
+                Type.getMethodDescriptor(Type.getType(Object[].class), TYPE_SMO, TYPE_TC, Type.getType(Class.class)));
+        }
+        else {
+            Label isntWrapped = new Label(), done = new Label();
+            mv.visitInsn(Opcodes.DUP);
+            mv.visitVarInsn(Opcodes.ALOAD, c.tcLoc);
+            mv.visitMethodInsn(Opcodes.INVOKESTATIC, TYPE_OPS.getInternalName(), "decont", Type.getMethodDescriptor(TYPE_SMO, TYPE_SMO, TYPE_TC));
+            mv.visitTypeInsn(Opcodes.INSTANCEOF, Type.getType(JavaObjectWrapper.class).getInternalName());
+            mv.visitJumpInsn(Opcodes.IFEQ, isntWrapped);
+            mv.visitVarInsn(Opcodes.ALOAD, c.tcLoc);
+            // XXX: the secondary decont is a bit awkward, but storing to the stack doesn't seem to work out
+            mv.visitMethodInsn(Opcodes.INVOKESTATIC, TYPE_OPS.getInternalName(), "decont", Type.getMethodDescriptor(TYPE_SMO, TYPE_SMO, TYPE_TC));
             mv.visitMethodInsn(Opcodes.INVOKESTATIC, "org/perl6/nqp/runtime/BootJavaInterop$RuntimeSupport", "unboxJava", Type.getMethodDescriptor(TYPE_OBJ, TYPE_SMO));
             mv.visitTypeInsn(Opcodes.CHECKCAST, Type.getInternalName(what));
+            mv.visitLabel(isntWrapped);
         }
     }
 
@@ -638,6 +770,7 @@ public class BootJavaInterop {
 
     /** Stores working information while building a class. */
     protected static class ClassContext {
+        public ClassContext() {}
         /** The ASM class writer. */
         public ClassWriter cv;
         /** The new class' internal name. */
@@ -797,6 +930,7 @@ public class BootJavaInterop {
 
     /** Working information for a method under construction. */
     protected static class MethodContext {
+        public MethodContext() { }
         /** The owning incomplete class. */
         public ClassContext cc;
         /** The ASM method writer. */
