@@ -19,6 +19,9 @@ public class ExceptionHandling {
     public static final int EX_CAT_WARN = 64;
     public static final int EX_CAT_SUCCEED = 128;
     public static final int EX_CAT_LABELED = 4096;
+    public static final int EX_CAT_AWAIT = 8192;
+    public static final int EX_CAT_EMIT = 16384;
+    public static final int EX_CAT_DONE = 32768;
     
     /* Exception handler kinds. */
     public static final int EX_UNWIND_SIMPLE = 0;
