@@ -13,11 +13,11 @@ public class IndyBootstrap {
             MethodType resType = MethodType.methodType(SixModelObject.class,
                     MutableCallSite.class, String.class, int.class, ThreadContext.class);
             MethodHandle res = caller.findStatic(IndyBootstrap.class, "wvalResolve_noa", resType);
-            
+
             /* Create a mutable callsite, and curry the resolver with it. */
             MutableCallSite cs = new MutableCallSite(type);
             cs.setTarget(MethodHandles.insertArguments(res, 0, cs));
-            
+
             /* Produce callsite; it'll be updated with the resolved WVal upon the
              * first invocation. */
             return cs;
@@ -26,11 +26,11 @@ public class IndyBootstrap {
             throw new RuntimeException(e);
         }
     }
-    
+
     public static SixModelObject wvalResolve_noa(MutableCallSite cs, String sc, int idx, ThreadContext tc) {
         /* Look up the WVal. */
-        SixModelObject res = tc.gc.scs.get(sc).root_objects.get(idx);
-        
+        SixModelObject res = tc.gc.scs.get(sc).getObject(idx);
+
         /* Update this callsite, so that we never run the lookup again and instead
          * just always use the resolved object. Discards incoming arguments, as
          * they are no longer needed. */
@@ -38,19 +38,19 @@ public class IndyBootstrap {
             cs.setTarget(MethodHandles.dropArguments(
                         MethodHandles.constant(SixModelObject.class, res),
                         0, String.class, int.class, ThreadContext.class));
-        
+
         /* Hand back the resulting object, for this first call. */
         return res;
     }
-    
-    public static CallSite subcall_noa(Lookup caller, String _, MethodType type) {
+
+    public static CallSite subcall_noa(Lookup caller, String s, MethodType type) {
         try {
             /* Look up subcall resolver method. */
             MethodType resType = MethodType.methodType(void.class,
                     Lookup.class, MutableCallSite.class, String.class,
                     int.class, ThreadContext.class, Object[].class);
             MethodHandle res = caller.findStatic(IndyBootstrap.class, "subcallResolve_noa", resType);
-            
+
             /* Create a mutable callsite, and curry the resolver with it and
              * the sub name. */
             MutableCallSite cs = new MutableCallSite(type);
@@ -58,7 +58,7 @@ public class IndyBootstrap {
                 .insertArguments(res, 0, caller, cs)
                 .asCollector(Object[].class, type.parameterCount() - 3)
                 .asType(type));
-            
+
             /* Produce callsite; it'll be updated with the resolved call upon the
              * first invocation. */
             return cs;
@@ -67,28 +67,30 @@ public class IndyBootstrap {
             throw new RuntimeException(e);
         }
     }
-    
+
     public static void subcallResolve_noa(Lookup caller, MutableCallSite cs, String name, int csIdx, ThreadContext tc, Object... args) {
         /* Locate the thing to call. */
         SixModelObject invokee = Ops.getlex(name, tc);
-        
+        if (invokee == null)
+            throw ExceptionHandling.dieInternal(tc, "Can not invoke object '" + name + "'");
+
         /* Don't update callsite in cases where it's not safe. */
         boolean shared = tc.curFrame.codeRef.staticInfo.compUnit.shared;
         if (invokee.st != null && invokee.st.ContainerSpec != null) {
             invokee = Ops.decont(invokee, tc);
             shared = true;
         }
-        
+
         /* Resolve callsite descriptor. */
         CallSiteDescriptor csd = csIdx >= 0
             ? tc.curFrame.codeRef.staticInfo.compUnit.callSites[csIdx]
             : Ops.emptyCallSite;
         CallSiteDescriptor csdOrig = csd;
-        
+
         /* If it's lexotic, then resolve to something to do the throwing. */
         if (invokee instanceof LexoticInstance)
             throw setLexoticTarget(caller, cs, tc, invokee, csd, args, shared);
-        
+
         /* Otherwise, get the code ref. */
         CodeRef cr;
         if (invokee instanceof CodeRef) {
@@ -106,7 +108,7 @@ public class IndyBootstrap {
                 args = tc.flatArgs;
             }
         }
-        
+
         /* Now need to adapt to the target callsite by binding the CodeRef
          * and callsite with what they've been resolved to. Don't do it if
          * it's a compiler stub, though. */
@@ -127,7 +129,7 @@ public class IndyBootstrap {
                 throw ExceptionHandling.dieInternal(tc, t);
             }
         }
-        
+
         /* Make the sub call directly for this initial call. */
         try {
             ArgsExpectation.invokeByExpectation(tc, cr, csd, args);
@@ -139,9 +141,12 @@ public class IndyBootstrap {
             ExceptionHandling.dieInternal(tc, e);
         }
     }
-    
-    public static void subInvoker(MethodHandle mh, String name, CallSiteDescriptor csd, ThreadContext tc, Object[] args) throws Throwable { 
+
+    public static void subInvoker(MethodHandle mh, String name, CallSiteDescriptor csd, ThreadContext tc, Object[] args) throws Throwable {
         SixModelObject invokee = Ops.getlex(name, tc);
+        if (invokee == null)
+            throw ExceptionHandling.dieInternal(tc, "Can not invoke object '" + name + "'");
+
         CodeRef cr;
         if (invokee instanceof CodeRef) {
             cr = (CodeRef)invokee;
@@ -160,15 +165,15 @@ public class IndyBootstrap {
         }
         ArgsExpectation.invokeByExpectation(tc, cr, csd, args);
     }
-    
-    public static CallSite subcallstatic_noa(Lookup caller, String _, MethodType type) {
+
+    public static CallSite subcallstatic_noa(Lookup caller, String s, MethodType type) {
         try {
             /* Look up subcall resolver method. */
             MethodType resType = MethodType.methodType(void.class,
                     Lookup.class, MutableCallSite.class, String.class,
                     int.class, ThreadContext.class, Object[].class);
             MethodHandle res = caller.findStatic(IndyBootstrap.class, "subcallstaticResolve_noa", resType);
-            
+
             /* Create a mutable callsite, and curry the resolver with it and
              * the sub name. */
             MutableCallSite cs = new MutableCallSite(type);
@@ -176,7 +181,7 @@ public class IndyBootstrap {
                 .insertArguments(res, 0, caller, cs)
                 .asCollector(Object[].class, type.parameterCount() - 3)
                 .asType(type));
-            
+
             /* Produce callsite; it'll be updated with the resolved call upon the
              * first invocation. */
             return cs;
@@ -185,28 +190,30 @@ public class IndyBootstrap {
             throw new RuntimeException(e);
         }
     }
-    
+
     public static void subcallstaticResolve_noa(Lookup caller, MutableCallSite cs, String name, int csIdx, ThreadContext tc, Object... args) {
         /* Locate the thing to call. */
         SixModelObject invokee = Ops.getlex(name, tc);
-        
+        if (invokee == null)
+            throw ExceptionHandling.dieInternal(tc, "Can not invoke object '" + name + "'");
+
         /* Don't update callsite in cases where it's not safe. */
         boolean shared = tc.curFrame.codeRef.staticInfo.compUnit.shared;
         if (invokee.st != null && invokee.st.ContainerSpec != null) {
             invokee = Ops.decont(invokee, tc);
             shared = true;
         }
-        
+
         /* Resolve callsite descriptor. */
         CallSiteDescriptor csd = csIdx >= 0
             ? tc.curFrame.codeRef.staticInfo.compUnit.callSites[csIdx]
             : Ops.emptyCallSite;
         CallSiteDescriptor csdOrig = csd;
-        
+
         /* If it's lexotic, then resolve to something to do the throwing. */
         if (invokee instanceof LexoticInstance)
             throw setLexoticTarget(caller, cs, tc, invokee, csd, args, shared);
-        
+
         /* Otherwise, get the code ref. */
         CodeRef cr;
         if (invokee instanceof CodeRef) {
@@ -225,7 +232,7 @@ public class IndyBootstrap {
                 shared = true;
             }
         }
-        
+
         /* Now need to adapt to the target callsite by binding the CodeRef
          * and callsite with what they've been resolved to. Don't do it if
          * it's a compiler stub, though. */
@@ -282,7 +289,7 @@ public class IndyBootstrap {
                 throw ExceptionHandling.dieInternal(tc, t);
             }
         }
-        
+
         /* Make the sub call directly for this initial call. */
         try {
             ArgsExpectation.invokeByExpectation(tc, cr, csd, args);
@@ -294,11 +301,11 @@ public class IndyBootstrap {
             ExceptionHandling.dieInternal(tc, e);
         }
     }
-    
-    public static void substaticInvoker(CallSiteDescriptor csd, CodeRef cr, ThreadContext tc, Object[] args) throws Throwable { 
+
+    public static void substaticInvoker(CallSiteDescriptor csd, CodeRef cr, ThreadContext tc, Object[] args) throws Throwable {
         ArgsExpectation.invokeByExpectation(tc, cr, csd, args);
     }
-    
+
     private static LexoticException setLexoticTarget(Lookup caller, MutableCallSite cs,
             ThreadContext tc, SixModelObject invokee, CallSiteDescriptor csd,
             Object[] args, boolean shared) {
@@ -370,43 +377,43 @@ public class IndyBootstrap {
         }
         return throwee;
     }
-    
-    public static void lexotic_o_noa(long target, String _, int __, ThreadContext tc, SixModelObject arg) {
+
+    public static void lexotic_o_noa(long target, String s, int i, ThreadContext tc, SixModelObject arg) {
         LexoticException throwee = tc.theLexotic;
         throwee.target = target;
         throwee.payload = arg;
         throw throwee;
     }
-    
-    public static void lexotic_i_noa(long target, SixModelObject boxType, String _, int __, ThreadContext tc, long arg) {
+
+    public static void lexotic_i_noa(long target, SixModelObject boxType, String s, int i, ThreadContext tc, long arg) {
         LexoticException throwee = tc.theLexotic;
         throwee.target = target;
         throwee.payload = Ops.box_i(arg, boxType, tc);
         throw throwee;
     }
-    
-    public static void lexotic_n_noa(long target, SixModelObject boxType, String _, int __, ThreadContext tc, double arg) {
+
+    public static void lexotic_n_noa(long target, SixModelObject boxType, String s, int i, ThreadContext tc, double arg) {
         LexoticException throwee = tc.theLexotic;
         throwee.target = target;
         throwee.payload = Ops.box_n(arg, boxType, tc);
         throw throwee;
     }
-    
-    public static void lexotic_s_noa(long target, SixModelObject boxType, String _, int __, ThreadContext tc, String arg) {
+
+    public static void lexotic_s_noa(long target, SixModelObject boxType, String s, int i, ThreadContext tc, String arg) {
         LexoticException throwee = tc.theLexotic;
         throwee.target = target;
         throwee.payload = Ops.box_s(arg, boxType, tc);
         throw throwee;
     }
-    
-    public static CallSite indcall_noa(Lookup caller, String _, MethodType type) {
+
+    public static CallSite indcall_noa(Lookup caller, String s, MethodType type) {
         try {
             /* Look up indirect call invoker method. */
             MethodType resType = MethodType.methodType(void.class,
                     MutableCallSite.class, int.class, ThreadContext.class,
                     SixModelObject.class, Object[].class);
             MethodHandle res = caller.findStatic(IndyBootstrap.class, "indcallInvoker_noa", resType);
-            
+
             /* Create a mutable callsite, and curry the resolver with it and
              * the sub name. */
             MutableCallSite cs = new MutableCallSite(type);
@@ -414,7 +421,7 @@ public class IndyBootstrap {
                 .insertArguments(res, 0, cs)
                 .asCollector(Object[].class, type.parameterCount() - 3)
                 .asType(type));
-            
+
             /* Produce callsite. */
             return cs;
         }
@@ -422,14 +429,14 @@ public class IndyBootstrap {
             throw new RuntimeException(e);
         }
     }
-    
+
     public static void indcallInvoker_noa(MutableCallSite cs, int csIdx,
             ThreadContext tc, SixModelObject invokee, Object... args) {
         /* Resolve callsite descriptor. */
         CallSiteDescriptor csd = csIdx >= 0
             ? tc.curFrame.codeRef.staticInfo.compUnit.callSites[csIdx]
             : Ops.emptyCallSite;
-        
+
         /* Get the code ref. */
         CodeRef cr;
         invokee = Ops.decont(invokee, tc);
@@ -450,7 +457,7 @@ public class IndyBootstrap {
                 args = tc.flatArgs;
             }
         }
-        
+
         /* Make the call. */
         try {
             ArgsExpectation.invokeByExpectation(tc, cr, csd, args);
@@ -462,15 +469,15 @@ public class IndyBootstrap {
             ExceptionHandling.dieInternal(tc, e);
         }
     }
-    
-    public static CallSite methcall_noa(Lookup caller, String _, MethodType type) {
+
+    public static CallSite methcall_noa(Lookup caller, String s, MethodType type) {
         try {
             /* Look up methcall resolver method. */
             MethodType resType = MethodType.methodType(void.class,
                     Lookup.class, MutableCallSite.class, String.class,
                     int.class, ThreadContext.class, Object[].class);
             MethodHandle res = caller.findStatic(IndyBootstrap.class, "methcallResolve_noa", resType);
-            
+
             /* Create a mutable callsite, and curry the resolver with it and
              * the method name. */
             MutableCallSite cs = new MutableCallSite(type);
@@ -478,7 +485,7 @@ public class IndyBootstrap {
                 .insertArguments(res, 0, caller, cs)
                 .asCollector(Object[].class, type.parameterCount() - 3)
                 .asType(type));
-            
+
             /* Produce callsite; it'll build up a PIC over various polymorphic
              * invocations. */
             return cs;
@@ -487,7 +494,7 @@ public class IndyBootstrap {
             throw new RuntimeException(e);
         }
     }
-    
+
     public static void methcallResolve_noa(Lookup caller, MutableCallSite cs, String name, int csIdx,
             ThreadContext tc, Object... args) {
         /* Resolve callsite descriptor. */
@@ -503,7 +510,7 @@ public class IndyBootstrap {
         SixModelObject invokee = Ops.findmethod(invocant, name, tc);
         if (invokee == null)
             throw ExceptionHandling.dieInternal(tc,
-                "Method '" + name + "' not found for invocant of class '" + Ops.typeName(invocant, tc) + "'"); 
+                "Method '" + name + "' not found for invocant of class '" + Ops.typeName(invocant, tc) + "'");
         CodeRef cr;
         if (invokee instanceof CodeRef) {
             cr = (CodeRef)invokee;
@@ -521,7 +528,7 @@ public class IndyBootstrap {
                 shared = true;
             }
         }
-        
+
         /* If not shared, then we'll optimize on the assumption that most
          * method callsites are monomorphic. */
         if (!cr.isCompilerStub && !shared) {
@@ -555,7 +562,7 @@ public class IndyBootstrap {
             ExceptionHandling.dieInternal(tc, e);
         }
     }
-    
+
     public static void methcallCacheMono_noa(String name, CallSiteDescriptor csd,
             STable assumedST, CodeRef assumedCR, ThreadContext tc, Object... args) {
         /* Try to resolve method to a coderef. */
@@ -568,8 +575,8 @@ public class IndyBootstrap {
             SixModelObject invokee = Ops.findmethod(invocant, name, tc);
             if (invokee == null)
                 throw ExceptionHandling.dieInternal(tc,
-                    "Method '" + name + "' not found for invocant of class '" + Ops.typeName(invocant, tc) + "'"); 
-            
+                    "Method '" + name + "' not found for invocant of class '" + Ops.typeName(invocant, tc) + "'");
+
             if (invokee instanceof CodeRef) {
                 cr = (CodeRef)invokee;
             }
@@ -598,15 +605,15 @@ public class IndyBootstrap {
             ExceptionHandling.dieInternal(tc, e);
         }
     }
-    
-    public static CallSite indmethcall_noa(Lookup caller, String _, MethodType type) {
+
+    public static CallSite indmethcall_noa(Lookup caller, String s, MethodType type) {
         try {
             /* Look up methcall invoker method. */
             MethodType resType = MethodType.methodType(void.class,
                     MutableCallSite.class, int.class,
                     ThreadContext.class, String.class, Object[].class);
             MethodHandle res = caller.findStatic(IndyBootstrap.class, "indmethcallInvoker_noa", resType);
-            
+
             /* Create a mutable callsite, and curry the resolver with it and
              * the method name. */
             MutableCallSite cs = new MutableCallSite(type);
@@ -614,7 +621,7 @@ public class IndyBootstrap {
                 .insertArguments(res, 0, cs)
                 .asCollector(Object[].class, type.parameterCount() - 3)
                 .asType(type));
-            
+
             /* Produce callsite. */
             return cs;
         }
@@ -622,14 +629,14 @@ public class IndyBootstrap {
             throw new RuntimeException(e);
         }
     }
-    
+
     public static void indmethcallInvoker_noa(MutableCallSite cs, int csIdx,
             ThreadContext tc, String name, Object... args) {
         /* Resolve callsite descriptor. */
         CallSiteDescriptor csd = csIdx >= 0
             ? tc.curFrame.codeRef.staticInfo.compUnit.callSites[csIdx]
             : Ops.emptyCallSite;
-        
+
         /* Try to resolve method to a coderef. */
         SixModelObject invocant = (SixModelObject)args[0];
         SixModelObject invokee = Ops.findmethod(invocant, name, tc);
