@@ -11,6 +11,8 @@ import org.perl6.nqp.sixmodel.reprs.CArray;
 import org.perl6.nqp.sixmodel.reprs.CPointer;
 import org.perl6.nqp.sixmodel.reprs.CStr;
 import org.perl6.nqp.sixmodel.reprs.CStruct;
+import org.perl6.nqp.sixmodel.reprs.CPPStruct;
+import org.perl6.nqp.sixmodel.reprs.CUnion;
 import org.perl6.nqp.sixmodel.reprs.IOHandle;
 import org.perl6.nqp.sixmodel.reprs.JavaWrap;
 import org.perl6.nqp.sixmodel.reprs.KnowHOWAttribute;
@@ -36,6 +38,8 @@ import org.perl6.nqp.sixmodel.reprs.Semaphore;
 import org.perl6.nqp.sixmodel.reprs.ConcBlockingQueue;
 import org.perl6.nqp.sixmodel.reprs.ConditionVariable;
 import org.perl6.nqp.sixmodel.reprs.AsyncTask;
+import org.perl6.nqp.sixmodel.reprs.NativeRef;
+import org.perl6.nqp.sixmodel.reprs.MultiDimArray;
 
 public class REPRRegistry {
     private static HashMap<String, Integer> reprIdMap = new HashMap<String, Integer>();
@@ -90,11 +94,15 @@ public class REPRRegistry {
         addREPR("CArray", new CArray());
         addREPR("CStr", new CStr());
         addREPR("CStruct", new CStruct());
+        addREPR("CPPStruct", new CPPStruct());
+        addREPR("CUnion", new CUnion());
         addREPR("VMThread", new VMThread());
         addREPR("ReentrantMutex", new ReentrantMutex());
         addREPR("Semaphore", new Semaphore());
         addREPR("ConcBlockingQueue", new ConcBlockingQueue());
         addREPR("ConditionVariable", new ConditionVariable());
         addREPR("AsyncTask", new AsyncTask());
+        addREPR("NativeRef", new NativeRef());
+        addREPR("MultiDimArray", new MultiDimArray());
     }
 }
