@@ -2,6 +2,8 @@ package org.perl6.nqp.sixmodel;
 
 import java.nio.ByteBuffer;
 import java.nio.ByteOrder;
+import java.util.ArrayList;
+import java.util.Map;
 
 import org.perl6.nqp.runtime.CallFrame;
 import org.perl6.nqp.runtime.CodeRef;
@@ -11,11 +13,11 @@ import org.perl6.nqp.sixmodel.reprs.VMHashInstance;
 
 public class SerializationReader {
     /* The current version of the serialization format. */
-    private final int CURRENT_VERSION = 8;
-    
+    private final int CURRENT_VERSION = 10;
+
     /* The minimum version of the serialization format. */
     private final int MIN_VERSION = 4;
-    
+
     /* Various sizes (in bytes). */
     private final int HEADER_SIZE               = 4 * 16;
     private final int DEP_TABLE_ENTRY_SIZE      = 8;
@@ -24,7 +26,7 @@ public class SerializationReader {
     private final int CLOSURES_TABLE_ENTRY_SIZE = 24;
     private final int CONTEXTS_TABLE_ENTRY_SIZE = 16;
     private final int REPOS_TABLE_ENTRY_SIZE    = 16;
-    
+
     /* Possible reference types we can serialize. */
     private final short REFVAR_NULL               = 1;
     private final short REFVAR_OBJECT             = 2;
@@ -38,7 +40,7 @@ public class SerializationReader {
     private final short REFVAR_VM_HASH_STR_VAR    = 10;
     private final short REFVAR_STATIC_CODEREF     = 11;
     private final short REFVAR_CLONED_CODEREF     = 12;
-    
+
     /* Starting state. */
     private ThreadContext tc;
     private SerializationContext sc;
@@ -47,10 +49,10 @@ public class SerializationReader {
     private int crCount;
     private CallFrame[] contexts;
     private ByteBuffer orig;
-    
+
     /* The version of the serialization format we're currently reading. */
     public int version;
-    
+
     /* Various table offsets and entry counts. */
     private int depTableOffset;
     private int depTableEntries;
@@ -67,13 +69,13 @@ public class SerializationReader {
     private int contextDataOffset;
     private int reposTableOffset;
     private int reposTableEntries;
-    
+
     /* Serialization contexts we depend on. */
     SerializationContext[] dependentSCs;
-    
+
     /* The object we're currently deserializing. */
     SixModelObject curObject;
-    
+
     public SerializationReader(ThreadContext tc, SerializationContext sc,
             String[] sh, CodeRef[] cr, int crCount, ByteBuffer orig) {
         this.tc = tc;
@@ -83,49 +85,48 @@ public class SerializationReader {
         this.crCount = crCount;
         this.orig = orig;
     }
-    
+
     public void deserialize() {
         // Serialized data is always little endian.
         orig.order(ByteOrder.LITTLE_ENDIAN);
-        
+
         // Split the input into the various segments.
         checkAndDisectInput();
         resolveDependencies();
-        
+
         // Put code refs in place.
         for (int i = 0; i < crCount; i++) {
             cr[i].isStaticCodeRef = true;
             cr[i].sc = sc;
-            sc.root_codes.add(cr[i]);
+            sc.addCodeRef(cr[i]);
         }
-        
-        // Handle any reposessions.
-        sc.root_stables.ensureCapacity(stTableEntries);
-        for (int i = 0; i < stTableEntries; i++)
-            sc.root_stables.add(null);
-        sc.root_objects.ensureCapacity(objTableEntries);
-        for (int i = 0; i < objTableEntries; i++)
-            sc.root_objects.add(null);
+
+        // Handle any STable repossessions, then stub STables.
+        sc.initSTableList(stTableEntries);
         if (reposTableEntries > 0)
-            repossess();
-        
-        // Stub all of the STables and objects.
+            repossess(1);
         stubSTables();
+
+        // Handle any object repossessions, then stub objects.
+        sc.initObjectList(objTableEntries);
+        if (reposTableEntries > 0)
+            repossess(0);
         stubObjects();
-        
+
         // Do first step of deserializing any closures.
         deserializeClosures();
-        
+
         // Second passes over STables and objects.
         deserializeSTables();
         deserializeObjects();
-        
+
         // Finish up contexts and closures.
         deserializeContexts();
         attachClosureOuters(crCount);
         attachContextOuters();
+        fixupContextOuters();
     }
-    
+
     /* Checks the header looks sane and all of the places it points to make sense.
      * Also disects the input string into the tables and data segments and populates
      * the reader data structure more fully. */
@@ -153,7 +154,7 @@ public class SerializationReader {
         prov_pos += depTableEntries * DEP_TABLE_ENTRY_SIZE;
         if (prov_pos > data_len)
             throw new RuntimeException("Corruption detected (dependencies table overruns end of data)");
-        
+
         /* Get size and location of STables table. */
         stTableOffset = orig.getInt();
         stTableEntries = orig.getInt();
@@ -187,7 +188,7 @@ public class SerializationReader {
         prov_pos = objDataOffset;
         if (prov_pos > data_len)
             throw new RuntimeException("Corruption detected (objects data starts after end of data)");
-        
+
         /* Get size and location of closures table. */
         closureTableOffset = orig.getInt();
         closureTableEntries = orig.getInt();
@@ -196,7 +197,7 @@ public class SerializationReader {
         prov_pos = closureTableOffset + closureTableEntries * CLOSURES_TABLE_ENTRY_SIZE;
         if (prov_pos > data_len)
             throw new RuntimeException("Corruption detected (Closures table overruns end of data)");
-        
+
         /* Get size and location of contexts table. */
         contextTableOffset = orig.getInt();
         contextTableEntries = orig.getInt();
@@ -205,7 +206,7 @@ public class SerializationReader {
         prov_pos = contextTableOffset + contextTableEntries * CONTEXTS_TABLE_ENTRY_SIZE;
         if (prov_pos > data_len)
             throw new RuntimeException("Corruption detected (contexts table overruns end of data)");
-        
+
         /* Get location of contexts data. */
         contextDataOffset = orig.getInt();
         if (contextDataOffset < prov_pos)
@@ -223,7 +224,7 @@ public class SerializationReader {
         if (prov_pos > data_len)
             throw new RuntimeException("Corruption detected (repossessions table overruns end of data)");
     }
-    
+
     private void resolveDependencies() {
         dependentSCs = new SerializationContext[depTableEntries];
         orig.position(depTableOffset);
@@ -240,73 +241,83 @@ public class SerializationReader {
             dependentSCs[i] = sc;
         }
     }
-    
+
     /* Repossess an object or STable. */
-    private void repossess() {
+    private void repossess(int chosenType) {
         for (int i = 0; i < reposTableEntries; i++) {
             /* Go to table row. */
             orig.position(reposTableOffset + i * REPOS_TABLE_ENTRY_SIZE);
-            
+
             /* Do appropriate type of repossession. */
             int repoType = orig.getInt();
+            if (repoType != chosenType)
+                continue;
             int objIdx = orig.getInt();
             int origSCIdx = orig.getInt();
             int origObjIdx = orig.getInt();
             if (repoType == 0) {
                 /* Get object to repossess. */
                 SerializationContext origSC = locateSC(origSCIdx);
-                SixModelObject origObj = origSC.root_objects.get(origObjIdx);
-                
-                /* Put it into objects root set at the apporpriate slot. */
-                sc.root_objects.set(objIdx, origObj);
-                
+                SixModelObject origObj = origSC.getObject(origObjIdx);
+
                 /* Ensure we aren't already trying to repossess the object. */
                 /* XXX TODO */
+
+                /* Put it into objects root set at the apporpriate slot. */
+                sc.addObject(origObj, objIdx);
+                origObj.sc = sc;
+
+                /* The object's STable may have changed as a result of the
+                 * repossession (perhaps due to mixing in to it), so put the
+                 * STable it should now have in place. */
+                orig.position(objTableOffset + objIdx * OBJECTS_TABLE_ENTRY_SIZE);
+                origObj.st = lookupSTable(orig.getInt(), orig.getInt());
             }
             else if (repoType == 1) {
                 /* Get STable to repossess. */
                 SerializationContext origSC = locateSC(origSCIdx);
-                STable origST = origSC.root_stables.get(origObjIdx);
-                
-                /* Put it into STables root set at the apporpriate slot. */
-                sc.root_stables.set(objIdx, origST);
-                
+                STable origST = origSC.getSTable(origObjIdx);
+
                 /* Ensure we aren't already trying to repossess the STable. */
                 /* XXX TODO */
+
+                /* Put it into STables root set at the apporpriate slot. */
+                sc.setSTable(objIdx, origST);
+                origST.sc = sc;
             }
             else {
                 throw new RuntimeException("Unknown repossession type");
             }
         }
     }
-    
+
     private void stubSTables() {
         for (int i = 0; i < stTableEntries; i++) {
             // May already have it, due to repossession.
-            if (sc.root_stables.get(i) != null)
+            if (sc.getSTable(i) != null)
                 continue;
-            
+
             // Look up representation.
             orig.position(stTableOffset + i * STABLES_TABLE_ENTRY_SIZE);
             REPR repr = REPRRegistry.getByName(lookupString(orig.getInt()));
-            
+
             // Create STable stub and add it to the root STable set.
             STable st = new STable(repr, null);
             st.sc = sc;
-            sc.root_stables.set(i, st);
+            sc.setSTable(i, st);
         }
     }
-    
+
     private void stubObjects() {
         for (int i = 0; i < objTableEntries; i++) {
             // May already have it, due to repossession.
-            if (sc.root_objects.get(i) != null)
+            if (sc.getObject(i) != null)
                 continue;
-            
+
             // Look up STable.
             orig.position(objTableOffset + i * OBJECTS_TABLE_ENTRY_SIZE);
             STable st = lookupSTable(orig.getInt(), orig.getInt());
-            
+
             // Now go by object flags.
             SixModelObject stubObj;
             orig.position(orig.position() + 4);
@@ -320,26 +331,26 @@ public class SerializationReader {
                 // Concrete object; defer to the REPR.
                 stubObj = st.REPR.deserialize_stub(tc, st);
             }
-            
+
             // Place object in SC root set.
             stubObj.sc = sc;
-            sc.root_objects.set(i, stubObj);
+            sc.addObject(stubObj, i);
         }
     }
-    
+
     private void deserializeClosures() {
         for (int i = 0; i < closureTableEntries; i++) {
             /* Seek to the closure's table row. */
             orig.position(closureTableOffset + i * CLOSURES_TABLE_ENTRY_SIZE);
-            
+
             /* Resolve the reference to the static code object. */
             CodeRef staticCode = readCodeRef();
-            
+
             /* Clone it and add it to this SC's code refs list. */
             CodeRef closure = (CodeRef)staticCode.clone(tc);
             closure.sc = sc;
-            sc.root_codes.add(closure);
-            
+            sc.addCodeRef(closure);
+
             /* See if there's a code object we need to attach. */
             orig.position(orig.position() + 4);
             if (orig.getInt() != 0)
@@ -352,15 +363,15 @@ public class SerializationReader {
             // Seek to the right position in the data chunk.
             orig.position(stTableOffset + i * STABLES_TABLE_ENTRY_SIZE + 4);
             orig.position(stDataOffset + orig.getInt());
-            
+
             // Get the STable we need to deserialize into.
-            STable st = sc.root_stables.get(i); 
-            
+            STable st = sc.getSTable(i);
+
             // Read the HOW, WHAT and WHO.
             st.HOW = readObjRef();
             st.WHAT = readObjRef();
             st.WHO = readRef();
-            
+
             /* Method cache and v-table. */
             SixModelObject methodCache = readRef();
             if (methodCache != null)
@@ -368,7 +379,7 @@ public class SerializationReader {
             st.VTable = new SixModelObject[(int)orig.getLong()];
             for (int j = 0; j < st.VTable.length; j++)
                 st.VTable[j] = readRef();
-            
+
             /* Type check cache. */
             int tcCacheSize = (int)orig.getLong();
             if (tcCacheSize > 0) {
@@ -376,10 +387,10 @@ public class SerializationReader {
                 for (int j = 0; j < st.TypeCheckCache.length; j++)
                     st.TypeCheckCache[j] = readRef();
             }
-            
+
             /* Mode flags. */
             st.ModeFlags = (int)orig.getLong();
-            
+
             /* Boolification spec. */
             if (orig.getLong() != 0) {
                 st.BoolificationSpec = new BoolificationSpec();
@@ -401,7 +412,7 @@ public class SerializationReader {
                     throw new RuntimeException("Unable to deserialize old container spec format");
                 }
             }
-            
+
             /* Invocation spec. */
             if (version >= 5) {
                 if (orig.getLong() != 0) {
@@ -412,45 +423,70 @@ public class SerializationReader {
                     st.InvocationSpec.InvocationHandler = readRef();
                 }
             }
-            
+
             /* HLL stuff. */
             if (version >= 6) {
                 st.hllOwner = tc.gc.getHLLConfigFor(readStr());
                 st.hllRole = orig.getLong();
             }
 
+            /* Type parametricity. */
+            if (version >= 9) {
+                long paraFlag = orig.getLong();
+                /* If it's a parametric type... */
+                if (paraFlag == 1) {
+                    ParametricType pt = new ParametricType();
+                    pt.parameterizer = readRef();
+                    pt.lookup = new ArrayList<Map.Entry<SixModelObject, SixModelObject>>();
+                    st.parametricity = pt;
+                }
+                else if (paraFlag == 2) {
+                    ParameterizedType pt = new ParameterizedType();
+                    pt.parametricType = readObjRef();
+                    SixModelObject BOOTArray = tc.gc.BOOTArray;
+                    pt.parameters = BOOTArray.st.REPR.allocate(tc, BOOTArray.st);
+                    int elems = orig.getInt();
+                    for (int j = 0; j < elems; j++)
+                        pt.parameters.bind_pos_boxed(tc, j, readRef());
+                    st.parametricity = pt;
+                }
+                else if (paraFlag != 0) {
+                    throw new RuntimeException("Unknown STable parametricity flag");
+                }
+            }
+
             /* If the REPR has a function to deserialize representation data, call it. */
             st.REPR.deserialize_repr_data(tc, st, this);
         }
     }
-    
+
     private void deserializeObjects() {
         for (int i = 0; i < objTableEntries; i++) {
             // Can skip if it's a type object.
-            SixModelObject obj = sc.root_objects.get(i);
+            SixModelObject obj = sc.getObject(i);
             if (obj instanceof TypeObject)
                 continue;
-            
+
             // Seek reader to object data offset.
             orig.position(objTableOffset + i * OBJECTS_TABLE_ENTRY_SIZE + 8);
             orig.position(objDataOffset + orig.getInt());
-            
+
             // Complete the object's deserialization.
             this.curObject = obj;
             obj.st.REPR.deserialize_finish(tc, obj.st, this, obj);
             this.curObject = null;
         }
     }
-    
+
     private void deserializeContexts() {
         contexts = new CallFrame[contextTableEntries];
         for (int i = 0; i < contextTableEntries; i++) {
             /* Seek to the context's table row. */
             orig.position(contextTableOffset + i * CONTEXTS_TABLE_ENTRY_SIZE);
-            
+
             /* Resolve the reference to the static code object this context is for. */
             CodeRef staticCode = readCodeRef();
-            
+
             /* Create a context and set it up. */
             CallFrame ctx = new CallFrame();
             ctx.tc = tc;
@@ -464,10 +500,10 @@ public class SerializationReader {
                 ctx.nLex = new double[sci.nLexicalNames.length];
             if (sci.sLexicalNames != null)
                 ctx.sLex = new String[sci.sLexicalNames.length];
-            
+
             /* Set context data read position, and set current read buffer to the correct thing. */
             orig.position(contextDataOffset + orig.getInt());
-            
+
             /* Deserialize lexicals. */
             long syms = orig.getLong();
             for (long j = 0; j < syms; j++) {
@@ -484,21 +520,21 @@ public class SerializationReader {
                 else
                     throw new RuntimeException("Failed to deserialize lexical " + sym);
             }
-            
+
             /* Put context in place. */
             contexts[i] = ctx;
         }
     }
-    
+
     private void attachClosureOuters(int closureBaseIdx) {
         for (int i = 0; i < closureTableEntries; i++) {
             orig.position(closureTableOffset + i * CLOSURES_TABLE_ENTRY_SIZE + 8);
             int idx = orig.getInt();
             if (idx > 0)
-                sc.root_codes.get(closureBaseIdx + i).outer = contexts[idx - 1];
+                sc.getCodeRef(closureBaseIdx + i).outer = contexts[idx - 1];
         }
     }
-    
+
     private void attachContextOuters() {
         for (int i = 0; i < contextTableEntries; i++) {
             orig.position(contextTableOffset + i * CONTEXTS_TABLE_ENTRY_SIZE + 12);
@@ -507,7 +543,16 @@ public class SerializationReader {
                 contexts[i].outer = contexts[idx - 1];
         }
     }
-    
+
+    private void fixupContextOuters() {
+        for (int i = 0; i < contextTableEntries; i++) {
+            if (contexts[i].outer == null &&
+                contexts[i].codeRef.staticInfo.priorInvocation != null &&
+                contexts[i].codeRef.staticInfo.priorInvocation.outer != null)
+                contexts[i].outer = contexts[i].codeRef.staticInfo.priorInvocation.outer;
+        }
+    }
+
     public SixModelObject readRef() {
         short discrim = orig.getShort();
         int elems;
@@ -584,50 +629,50 @@ public class SerializationReader {
             throw new RuntimeException("Unimplemented case of read_ref");
         }
     }
-    
+
     public SixModelObject readObjRef() {
         SerializationContext sc = locateSC(orig.getInt());
         int idx = orig.getInt();
-        if (idx < 0 || idx >= sc.root_objects.size())
+        if (idx < 0 || idx >= sc.objectCount())
             throw new RuntimeException("Invalid SC object index " + idx);
-        return sc.root_objects.get(idx);
+        return sc.getObject(idx);
     }
-    
+
     public STable readSTableRef() {
         return lookupSTable(orig.getInt(), orig.getInt());
     }
-    
+
     public CodeRef readCodeRef() {
         SerializationContext sc = locateSC(orig.getInt());
         int idx = orig.getInt();
-        if (idx < 0 || idx >= sc.root_codes.size())
+        if (idx < 0 || idx >= sc.coderefCount())
             throw new RuntimeException("Invalid SC code index " + idx);
-        return sc.root_codes.get(idx);
+        return sc.getCodeRef(idx);
     }
-    
+
     public long readLong() {
         return orig.getLong();
     }
-    
+
     public int readInt32() {
         return orig.getInt();
     }
-    
+
     public double readDouble() {
         return orig.getDouble();
     }
-    
+
     public String readStr() {
         return lookupString(orig.getInt());
     }
 
     private STable lookupSTable(int scIdx, int idx) {
         SerializationContext sc = locateSC(scIdx);
-        if (idx < 0 || idx >= sc.root_stables.size())
+        if (idx < 0 || idx >= sc.stableCount())
             throw new RuntimeException("Invalid STable index (scIdx=" + scIdx + ",idx=" + idx + ")");
-        return sc.root_stables.get(idx);
+        return sc.getSTable(idx);
     }
-    
+
     private SerializationContext locateSC(int scIdx) {
         if (scIdx == 0)
             return sc;
@@ -635,7 +680,7 @@ public class SerializationReader {
             throw new RuntimeException("Invalid dependencies table index encountered (index " + scIdx + ")");
         return dependentSCs[scIdx - 1];
     }
-    
+
     private String lookupString(int idx) {
         if (idx >= sh.length)
             throw new RuntimeException("Attempt to read past end of string heap (index " + idx + ")");
