@@ -6,13 +6,19 @@ import org.perl6.nqp.runtime.ThreadContext;
 import org.perl6.nqp.sixmodel.SixModelObject;
 
 public class VMHashInstance extends SixModelObject {
+    public static final HashMap<String, SixModelObject> EMPTY_MAP = 
+            new HashMap<String, SixModelObject>();
+
     public HashMap<String, SixModelObject> storage;
-    
+
     public SixModelObject at_key_boxed(ThreadContext tc, String key) {
         return storage.get(key);
     }
     
     public void bind_key_boxed(ThreadContext tc, String key, SixModelObject value) {
+        if (storage == VMHashInstance.EMPTY_MAP) {
+            storage = new HashMap<String, SixModelObject>();
+        }
         storage.put(key, value);
     }
     
