@@ -12,6 +12,17 @@
 #define EDGE_CHARLIST_NEG      7
 #define EDGE_CODEPOINT_I       9
 #define EDGE_CODEPOINT_I_NEG   10
+#define EDGE_GENERIC_VAR       11
+#define EDGE_CHARRANGE         12
+#define EDGE_CHARRANGE_NEG     13
+#define EDGE_CODEPOINT_LL      14
+#define EDGE_CODEPOINT_I_LL    15
+#define EDGE_CODEPOINT_M       16
+#define EDGE_CODEPOINT_M_NEG   17
+#define EDGE_CODEPOINT_M_LL    18
+#define EDGE_CODEPOINT_IM      19
+#define EDGE_CODEPOINT_IM_NEG  20
+#define EDGE_CODEPOINT_IM_LL   21
 
 /* State entry. */
 typedef struct {
