@@ -124,6 +124,14 @@ typedef struct {
 /* This flag is set if we consider the method cche authoritative. */
 #define METHOD_CACHE_AUTHORITATIVE     4
 
+/* This STable mode flag is set if the type is parametric (and so can be
+ * parameterized). */
+#define SM_PARAMETRIC_TYPE             16
+
+/* This STable mode flag is set if the type is a parameterization of some
+ * parametric type. */
+#define SM_PARAMETERIZED_TYPE          32
+
 /* HLL type roles. */
 #define HLL_ROLE_NONE       0
 #define HLL_ROLE_INT        1
@@ -146,6 +154,27 @@ struct SixModel_STable {
     /* The meta-object. */
     PMC *HOW;
 
+    /* Parametricity. Mode flags indicate what, if any, of this union is valid. */
+    union {
+        struct {
+            /* The code object to use to produce a new parameterization. */
+            PMC *parameterizer;
+
+            /* Lookup table of existing parameterizations. For now, just a VM
+             * array with alternating pairs of [arg array], object. Could in
+             * the future we something lower level or hashy; we've yet to see
+             * how hot-path lookups end up being in reality. */
+            PMC *lookup;
+        } ric;
+        struct {
+            /* The type that we are a parameterization of. */
+            PMC *parametric_type;
+
+            /* Our type parameters. */
+            PMC *parameters;
+        } erized;
+    } paramet;
+
     /* The type-object. */
     PMC *WHAT;
 
@@ -383,7 +412,7 @@ struct SixModel_REPROps {
     INTVAL (*elems) (PARROT_INTERP, STable *st, void *data);
     
     /* Gets the storage specification for this representation. */
-    storage_spec (*get_storage_spec) (PARROT_INTERP, STable *st);
+    void (*get_storage_spec) (PARROT_INTERP, STable *st, storage_spec *spec);
     
     /* Handles an object changing its type. The representation is responsible
      * for doing any changes to the underlying data structure, and may reject
@@ -487,12 +516,19 @@ PMC * hllize(PARROT_INTERP, PMC *obj, INTVAL hll_id);
 typedef PMC * (*wrap_object_t)(PARROT_INTERP, void *obj);
 typedef PMC * (*create_stable_t)(PARROT_INTERP, REPROps *REPR, PMC *HOW);
 typedef INTVAL (* rf) (PARROT_INTERP, STRING *name, REPROps * (*reg) (PARROT_INTERP, wrap_object_t, create_stable_t));
+typedef INTVAL (* lf) (PARROT_INTERP, STRING *name);
 #define REGISTER_DYNAMIC_REPR(interp, name, reg_func) \
     ((rf) \
         VTABLE_get_pointer(interp, \
             VTABLE_get_pmc_keyed_str(interp, interp->root_namespace, \
                 Parrot_str_new_constant(interp, "_REGISTER_REPR"))))(interp, name, reg_func)
 
+#define LOOKUP_REPR_ID(interp, name) \
+    ((lf) \
+        VTABLE_get_pointer(interp, \
+            VTABLE_get_pmc_keyed_str(interp, interp->root_namespace, \
+                Parrot_str_new_constant(interp, "_LOOKUP_REPR_ID"))))(interp, name)
+
 /* Dynamic container configuration registration. */
 typedef void (*cspec_conf) (PARROT_INTERP, STRING *name, ContainerConfigurer *configurer);
 #define REGISTER_DYNAMIC_CONTAINER_CONFIG(interp, name, configurer) \
