@@ -12,6 +12,7 @@ our @EXPORT_OK = qw(sorry slurp system_or_die
                     git_checkout
                     verify_install gen_moar
                     github_url
+                    probe_node
                     gen_nqp gen_parrot);
 
 our $exe = $^O eq 'MSWin32' ? '.exe' : '';
@@ -20,7 +21,7 @@ our $bat = $^O eq 'MSWin32' ? '.bat' : '';
 our @required_parrot_files = qw(
     @bindir@/parrot@exe@
     @bindir@/pbc_to_exe@exe@
-    @bindir@/ops2c@exe@
+    @bindir@/@ops2c@@exe@
     @libdir@@versiondir@/tools/build/pmc2c.pl
     @srcdir@@versiondir@/pmc
     @includedir@@versiondir@/pmc
@@ -81,7 +82,7 @@ sub read_config {
     local $_;
     for my $file (@config_src) {
         no warnings;
-        if (open my $CONFIG, '-|', "$file --show-config") {
+        if (open my $CONFIG, '-|', "\"$file\" --show-config") {
             while (<$CONFIG>) {
                 if (/^([^\s=]+)=(.*)/) { $config{$1} = $2 }
             }
@@ -139,6 +140,7 @@ END
         last if %config;
     }
     unlink('parrot-config.pir');
+    $config{'parrot::ops2c'} = 'ops2c' unless exists $config{'parrot::ops2c'};
     return %config;
 }
 
@@ -210,11 +212,18 @@ sub git_checkout {
     my $dir  = shift;
     my $checkout = shift;
     my $pushurl = shift;
+    my $depth = shift;
+    my $reference = shift;
     my $pwd = cwd();
 
     # get an up-to-date repository
     if (! -d $dir) {
-        system_or_die('git', 'clone', $repo, $dir);
+        my @args = ('git', 'clone');
+        push @args, $reference if $reference ne '';
+        push @args, $depth if $depth ne '';
+        push @args, $repo;
+        push @args, $dir;
+        system_or_die(@args);
         chdir($dir);
         system('git', 'config', 'remote.origin.pushurl', $pushurl)
             if defined $pushurl && $pushurl ne $repo;
@@ -316,6 +325,8 @@ sub gen_nqp {
             github_url($git_protocol, 'perl6', 'nqp'),
             'nqp', $nqp_want,
             github_url('ssh', 'perl6', 'nqp'),
+            $options{'git-depth'} ? "--depth=$options{'git-depth'}" : '',
+            $options{'git-reference'} ? "--reference=$options{'git-reference'}/nqp" : '',
         );
     }
 
@@ -368,6 +379,8 @@ sub gen_parrot {
             github_url($git_protocol, 'parrot', 'parrot'),
             'parrot', $gen_parrot,
             github_url('ssh', 'parrot', 'parrot'),
+            $options{'git-depth'} ? "--depth=$options{'git-depth'}" : '',
+            $options{'git-reference'} ? "--reference=$options{'git-reference'}/parrot" : '',
         );
         $par_ok = $par_have eq $par_repo;
     }
@@ -376,6 +389,8 @@ sub gen_parrot {
             github_url($git_protocol, 'parrot', 'parrot'),
             'parrot', $par_want,
             github_url('ssh', 'parrot', 'parrot'),
+            $options{'git-depth'} ? "--depth=$options{'git-depth'}" : '',
+            $options{'git-reference'} ? "--reference=$options{'git-reference'}/parrot" : '',
         );
     }
 
@@ -422,13 +437,16 @@ sub gen_moar {
     my %options  = @_;
 
     my $prefix     = $options{'prefix'} || cwd()."/install";
+    my $sdkroot    = $options{'sdkroot'} || '';
     my $gen_moar   = $options{'gen-moar'};
     my @opts       = @{ $options{'moar-option'} || [] };
     push @opts, "--optimize";
     my $startdir   = cwd();
     my $git_protocol = $options{'git-protocol'} || 'https';
-
-    my $moar_exe   = "$prefix/bin/moar$exe";
+    my $moar_exe   = $options{'with-moar'} || (
+        $sdkroot
+            ? File::Spec->catfile( $sdkroot, $prefix, 'bin', "moar$exe" )
+            : File::Spec->catfile( $prefix, 'bin', "moar$exe" ));
     my $moar_have  = qx{ $moar_exe --version };
     if ($moar_have) {
         $moar_have = $moar_have =~ /version (\S+)/ ? $1 : undef;
@@ -449,6 +467,8 @@ sub gen_moar {
         github_url($git_protocol, 'MoarVM', 'MoarVM'),
         'MoarVM', $gen_moar || $moar_want,
         github_url('ssh', 'MoarVM', 'MoarVM'),
+        $options{'git-depth'} ? "--depth=$options{'git-depth'}" : '',
+        $options{'git-reference'} ? "--reference=$options{'git-reference'}/MoarVM" : '',
     );
 
     unless (cmp_rev($moar_repo, $moar_want) >= 0) {
@@ -482,5 +502,17 @@ sub github_url {
     }
 }
 
+sub probe_node {
+    # Debian ships a 'node' binary that is related to amateur radio.
+    # the javascript thingy is called 'nodejs' there
+    for my $binary (qw/node nodejs /) {
+        my $version_str = qx/$binary -v 2>&1/;
+        if ($version_str =~ /^v\d/) {
+            return $binary;
+        }
+    }
+    return;
+}
+
 
 1;
